//
// GameX (Release 2 - Vectors / Matricies / CameraX)
// CameraX Class Header
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
//

#ifndef GAMEX_CAMERAX_DEF
	#define GAMEX_CAMERAX_DEF

	#include "gamex-matrix.hpp"

	#define DEG_TO_RAD			(3.141592/180)

	#define CAMERA_PERSPECT		0
	#define CAMERA_PARALLEL		1

	class CameraX {
	public:
		CameraX ();

		void SetFromPosition (Vector3DF &from);
		void SetFromAngles (Vector3DF &angs);
		void SetToPosition (Vector3DF &to);
		void SetUpVector (Vector3DF &up);
		void SetNearPlane (double n);
		void SetFarPlane (double f);
		void SetViewHeight (double h);
		void SetProjection (int proj_const);
		void SetWindow (int x1, int y1, int x2, int y2);

		void ViewTransform (Vector3DF &p, Vector4DF &q);
		void ProjectTransform (Vector4DF &q);
		void ScreenTransform (Vector4DF &q, Vector3DF &p);
		void CameraTransform (Vector3DF &p, Vector4DF &q);
		void WindowTransform (Vector4DF &q, int &x, int &y, double &z);
		void InverseTransform (Vector3DF &p, Vector4DF &q);
		
		void Project (Vector3DF &q, int &x, int &y, double &z);

		double GetHeight () {return height_dist;}
		double GetNear () {return near_dist;}
		double GetFar () {return far_dist;}
		Vector3DF &From () {return from_pos;}
		Vector3DF &To () {return to_pos;}
	private:
		void MakeDirVector ();			// Calculate direction (using to and from vectors)
		void MakeViewMatrix ();			// Calculate view matrix (using dir, from and side vectors)
		void MakeProjMatrix ();			// Calculate projection matrix (using proj. parameters)

		int proj_type;					// Camera projection type

		Vector3DF from_pos, to_pos;		// Camera To and From positions
										// Camera Tview vectors (N, V, and U)
		Vector3DF dir_vector, up_vector, side_vector;
		MatrixF view_matrix;			// Camera Tview matrix
										// Camera Tpersp parameters (d, f, and h)		
		double near_dist, far_dist, height_dist;	
		MatrixF proj_matrix;			// Camera Tpersp (also Tparallel or Tortho)
					
		MatrixF transform_matrix;		// Camera Transform (Tview * Tpersp)

		int cam_x1, cam_y1;				// Camera Window (Maps -1<x,y<1 to 0<x,y<win)
		int cam_x2, cam_y2;
		int cam_sx, cam_sy;
		int cam_cx, cam_cy;
	};

#endif
