//
// GameX - ImageX Class Header 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
										// **** Image includes
#include "gamex-buffer.hpp"
#include "gamex-file.hpp"

#ifndef IMAGEX_DEF
	#define	IMAGEX_DEF
										// **** Image Switches
	#define IMAGEX_DEBUG				// Image debugging (bounds, null data, etc.)
	//#define IMAGEX_SUPPORT_RAW			// Image support for RAW file load/save
	#define IMAGEX_SUPPORT_TIFF			// Image support for TIFF file load/save
	
	#define DRAW_UNDEF		0
	#define DRAW_IMAGE		1
	#define DRAW_MASKED		2
	#define DRAW_ALPHA		3
	#define DRAW_ADDED		4

	#define IMG_OK			0
	#define IMG_NOTREADY	1
	#define IMG_IGNORE		2
										// **** Image Pixel Channels
	#define CHAN_INDEXED		0	
	#define CHAN_RED			1
	#define CHAN_GREEN			2
	#define CHAN_BLUE			3
	#define CHAN_ALPHA			4
	#define CHAN_GRAY			5
										// **** Image Pixel-Format Information
	#define IMG_UNDEF			0		// Image information is undefined.
	#define IMG_ALPHA			1		// Image has an alpha channel (follows color data)
	#define IMG_MERGEALPHA		2		// Alpha channel is merged with data at pixel level (RGBA)	
	#define IMG_INDEXED			4		// Image is indexed
	#define IMG_LOWCOLOR		8		// Image is low color.	(BPP=4-4-4-[A])
	#define IMG_HIGHCOLOR		16		// Image is high color. (BPP=5-5-6-[A])
	#define IMG_TRUECOLOR		32		// Image is true color. (BPP=8-8-8-[A])
	#define IMG_MASKED			64		// Image uses a 0-mask (instead of alpha)

										// **** File Formats
	#define IMG_RAW_FORMAT		0		// RAW format
	#define IMG_TIFF_FORMAT		1		// TIFF format	


	#define TRUE				1
	#define FALSE				0

	#define DEGtoRAD	(3.141592/180)

	typedef unsigned char		ImageOps;	

	class ImageX {
	public:		
		ImageX ();										// Constructors
		ImageX (int xr, int yr, ImageOps ops);
		~ImageX ();										// Destructor
		
		int GetWidth (void)	{return m_xres;}
		int GetHeight (void) {return m_yres;}
		void Size (int xr, int yr, ImageOps ops);	// Construct Image
		void Create (int xr, int yr, int u);
		void Fill (int r, int g, int b);
		inline void TurnOn (ImageOps opt);					// Turn on a PixelFormat option
		inline void TurnOff (ImageOps opt);					// Turn off a PixelFormat option
		inline int GetBytesPerPixel (ImageOps ops);			// Get Bytes-Per-Pixel
		inline int GetFormat (ImageOps ops);				// Get Format 
		inline int GetAlpha (ImageOps ops);					// Get Alpha
		inline char GetUsage (void)	{return m_usage;}
		inline void SetUsage (int u) {m_usage = u;}
		void ConvertFormat (ImageOps pf);
		void ConvertAlphaToMask (void);
		void ConvertAlphaToBlend (void);
		void UseWith (int what);
		void SetPixel (int x, int y, int r, int g, int b);
		void GetPixel (int x, int y, int &r, int &g, int &b);

		void Rotate (ImageX *dest, float ang, float scale);
		void RotateImage (ImageX *dest, float ang, float scale);
		void RotateAlpha (ImageX *dest, float ang, float scale);

		XBYTE *m_data;										// Color Data
		XBYTE *m_alpha;										// Alpha Channel
		int m_xres, m_yres;									// Resolution
		char m_usage;
		ImageOps m_options;									// Pixel Format
		char status;										// Image Ready?
	};

	class ImageExt {
	public:
		ImageExt ();
		ImageExt (ImageX* pImg);
		~ImageExt ();
		void Load (char *filename, ImageX *img, int what);
	private:
		#ifdef IMAGEX_SUPPORT_TIFF
			void LoadTiff (char *filename, ImageX *img);
			void SaveTiff (char *filename, ImageX *img);
			void LoadTiffDirectory (File &tiff, Buffer &code);
			void LoadTiffEntry (File &tiff, Buffer &code);
			void LoadTiffStrips (File &tiff, Buffer &code);
			void LoadTiffData (File &tiff, Buffer &code, unsigned long count, unsigned long offset, int row);
			void SaveTiffDirectory (File &tiff, Buffer &code);
			void SaveTiffEntry (File &tiff, Buffer &code, unsigned int tag);
			void SaveTiffExtras (File &tiff, Buffer &code, unsigned int tag);
			void SaveTiffData (File &tiff, Buffer &code);			
		#endif

		ImageX*				m_pImg;					// Pointer to an ImageX		
		char				m_filename[100];		// Filename of ImageX
		int					m_status;				// Status of ImageX
		
		#ifdef IMAGEX_SUPPORT_RAW					// RAW Format Info
			int				m_bpp;
			int				m_channel;
			int				m_header;
		#endif

		#ifdef IMAGEX_SUPPORT_TIFF					//**** TIFF Format Info
			int				m_xres, m_yres;			// TIFF Resolution
			int				m_ops;					// TIFF Pixel-Format Options for ImageX
			int				m_mode;					// TIFF Mode
			int				m_alpha;				// TIFF Alpha Channel Present?
			int				m_num_chan;				// TIFF Number of Channels
			unsigned long	m_bpcoff;				// TIFF 
			int				m_bpc[5];				// TIFF Bits per Channel
			int				m_compress;				// TIFF Compressed?
			int				m_photo;
			unsigned long	m_num_strips;			// TIFF Number of Strips
			unsigned long	m_strip_offsets;		// TIFF Strip Offsets
			unsigned long	m_strip_counts;			// TIFF Strip Counts
			int				m_rps;					// TIFF Rows per Strip
			int				m_bpp;					// TIFF Bits per Pixel
			int				m_bpr;					// TIFF Bits per Row
		#endif
	};
#endif
