//
// GameX - WindowsDX Class Header 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
#ifndef WINDX_DEF
	#define WINDX_DEF

	#include <windows.h>
	#include <stdio.h>
	#include <ddraw.h>
	#include <dsound.h>

	#include "gamex-debug.hpp"
	#include "gamex-image.hpp"
	#include "gamex-sound.hpp"

	#define WINDX_DEBUG						// Turns on WindowsDX debugging	

	#define GAMEX_NO_INIT				0
	#define GAMEX_INITCLASS_FAILED		1
	#define GAMEX_INITCLASS_OK			2
	#define GAMEX_INITWINTEMP_FAILED	3
	#define GAMEX_INITWINTEMP_OK		4
	#define GAMEX_INITWINREAL_FAILED	5
	#define GAMEX_INITWINREAL_OK		6
	#define GAMEX_SHOWWIN_FAILED		7
	#define GAMEX_SHOWWIN_OK			8
	#define GAMEX_DDSTART_FAILED		9
	#define GAMEX_DDSTART_OK			10
	#define GAMEX_DDSETLEVEL_FAILED		11
	#define GAMEX_DDSETLEVEL_OK			12
	#define GAMEX_SETMODE_FAILED		13
	#define GAMEX_SETMODE_OK			14
	#define GAMEX_FRONTSURFACE_FAILED	15
	#define GAMEX_FRONTSURFACE_OK		16
	#define GAMEX_BACKSURFACE_FAILED	17
	#define GAMEX_BACKSURFACE_OK		18	
	#define GAMEX_READY					19
	#define GAMEX_LOCK_FAILED			20
	#define GAMEX_REQSIZE_FAILED		21
	#define GAMEX_BPP_FAILED			22
	#define GAMEX_SOUNDINIT_FAILED		23
	#define GAMEX_SOUNDCOOP_FAILED		24
	#define GAMEX_SOUNDCREATE_FAILED	25
	#define GAMEX_SOUNDMATCH_FAILED		26
	#define GAMEX_SOUNDCAPS_FAILED		27
	#define GAMEX_SOUNDBUFLEN_FAILED	28
	#define GAMEX_SOUNDLOST_FAIL		29
	#define GAMEX_SOUNDINVALID_FAIL		30
	#define GAMEX_SOUNDPARAM_FAIL		31
	#define GAMEX_SOUNDPRIOLEVEL_FAIL	32
	#define GAMEX_IMAGEUSE_FAILED		33

	#define WINDX_BUFSIZE			1000	// Keyboard Buffer Size

	#define MOUSE_POS_RATE	1.0
	#define MOUSE_LEFT		2
	#define MOUSE_MIDDLE	3
	#define MOUSE_RIGHT		4

	typedef	SoundX*		SoundXPtr;

	class WindowsDX {
	public:
		WindowsDX ();
		~WindowsDX (void);

		//**** Initialize GameX
		bool Initialize (int nWinFull, int xres, int yres);
		void Quit (void);
		
		//**** Interface Functions (cross-platform)
		// Request Functions
		void UseAlpha (void);		
		// Drawing Functions
		void SetView (int x1, int y1, int x2, int y2);			// Set viewport		
		bool DrawOpen ();										// Open graphics output
		int DrawClose ();										// Close graphics output
		void DrawClear (void);									// Clear viewport
		void DrawColor (int r, int g, int b);					// Select drawing color		
		inline int DrawPixel (int x, int y, int r, int g, int b)	{return (*FuncDrawPixel) (x,y,r,g,b);}	// Draw pixel
		inline int DrawImage (int x, int y, ImageX *img)			{return (*FuncDrawImage) (x,y,img);}	// Draw image
		inline int DrawMasked (int x, int y, ImageX *img)			{return (*FuncDrawMasked) (x,y,img);}	// Draw masked image
		inline int DrawAlpha (int x, int y, ImageX *img)			{return (*FuncDrawAlpha) (x,y,img);}	// Draw alpha image
		inline int DrawAdded (int x, int y, ImageX *img)			{return (*FuncDrawAdded) (x,y,img);}	// Draw added image
		int DrawBackground (int x, int y, ImageX *img);		

		void PlaySound (SoundX* snd, int how);
		void PlaySound (SoundX* snd, int how, float vol, float pan);
		void StopSound (SoundX* snd);
		void RewindSound (SoundX* snd);

		int DrawText (int x, int y, char *msg);
		void DrawBox (int x1, int y1, int x2, int y2, int r, int g, int b);
		int ViewUpdate (void);									// Updates view for the next frame		
		// Keyboard Input Functions
		int GetStatus (void)			{return win_status;}
		char GetKey (void);
		bool GetKeyDown (int n)			{return win_keydown[n];}
		int GetWidth ();
		int GetHeight ();
		int GetPitch ();
		void *GetScreen (void);
		HWND GetWindow ();		
		// Mouse Input Functions
		int GetMousePress ();
		int GetMouseX ();
		int GetMouseY ();
		float GetMouseDX ();
		float GetMouseDY ();

		int GetShift (int mask);
		int GetMax (int mask);

		//*** Windows Control Functions
		// Report a Problem Initializing
		void ReportProblem (void);		
		void ReportProblem (int problem);
		// Initialization & Clean Up Functions		
		void WindowInitInstance (HINSTANCE hThisInst, int nWinMode);	
		bool InitWindows (void);
		bool InitDirectX (void);
		bool InitSoundX (void);
		void DirectDrawError (int status);
		bool CleanUp (void);		
		// Internal State Control Functions
		void SetActivation (BOOL act);
		void SetPosition (int x, int y);		
		void SetMouseMove (int x, int y);
		void SetMousePress (int button);
		void SetMouseRelease ();
		void SetKey (char ch);
		void SetKeyDown (char ch)		{win_keydown[ch] = true;}
		void SetKeyUp (char ch)			{win_keydown[ch] = false;}
		
		// Clipping Functions
		void ClipTile (int x, int y, int xres, int yres, int &mx, int &my, int &offx, int &offy);
		bool ClipImage (int x, int y, ImageX *img, int &view_offset, int &img_offset, int &img_xs, int &img_ys);

		// Public Variables (for Static Draw Function Switching)
		char	*win_screen;
		int		win_ytable[1024];
		int		win_height, win_width, win_pitch;

	private:		
		//**** Private Functions		
		// Dispatched Drawing Functions
		int (*FuncDrawPixel) (int x, int y, int r, int g, int b);
		int (*FuncDrawImage) (int x, int y, ImageX *img);
		int (*FuncDrawMasked) (int x, int y, ImageX *img);
		int (*FuncDrawAlpha) (int x, int y, ImageX *img);
		int (*FuncDrawAdded) (int x, int y, ImageX *img);		
		
		// Windows & DirectX Specific Initialization
		bool WindowInitClass (void);								// Create a window class
		bool WindowInit (void);									// Create a main window
		bool WindowDraw (void);									// Draw window on screen
		bool DirectDrawInit (void);								// Initialize direct draw
		bool DirectSoundInit (void);
		bool DirectSoundFormat (void);
		bool DirectDrawSurfInit (void);							// Initialize direct draw surfaces
		bool DirectDrawCleanUp (void);							// Cleanup direct draw

		// Sound Functions
		void UpdateSound (SoundX* snd);
		int AddSound (SoundX* snd);

		HINSTANCE				win_hinst;	
		int						win_hmode;

		int						win_status; 
		
		int						win_request_full;
		int						win_request_xsize, win_request_ysize;

		int						win_fullscreen;	
		int						win_screenx, win_screeny;
		int						win_screenbpp;
		int						win_viewx1, win_viewy1;
		int						win_viewx2, win_viewy2;		
		int						win_size;
		int						win_widbits;
		int						win_xpos, win_ypos;		
		RECT					win_destrect;
		BOOL					win_active;

		HWND					win_hwnd;

		int						win_r, win_g, win_b;
		long int				win_rgb;		
		int						win_keys;
		char					win_keybuf[WINDX_BUFSIZE];
		bool					win_keydown[256];

		int						mouse_x, mouse_y;
		int						mouse_xl, mouse_yl;
		int						mouse_down, mouse_button;
		int						mouse_motion;

		unsigned short *		table_alpha;

		// Direct Draw 8.1
		//LPDIRECT3D8			dd_main;		// D3DDevice
        //LPDIRECT3DDEVICE8		dd_device;		// Rendering device
        //D3DCAPS8 *			dd_caps;		// Caps for the Rendering Device
		//LPD3DXSPRITE			dd_sprite;		// Sprites
		//LPDIRECT3DTEXTURE8	dd_texture;
		//RECT *				dd_rect;
		
		// Direct Draw 3.1		
		LPDIRECTDRAW			dd_main;
		LPDIRECTDRAWSURFACE		dd_front_buf;
		LPDIRECTDRAWSURFACE		dd_back_buf;

		// Direct Sound 3.1
		LPDIRECTSOUND			ds_main;
		int						ds_samples_per_sec;
		int						ds_bits_per_sample;
		int						ds_num_channels;

		bool					sound_support;

		int						num_sounds;
		int						max_sounds;
		int						num_playing;
		SoundXPtr*				sound_list;
		LPDIRECTSOUNDBUFFER*	ds_sound_list;
	};

	//*** Switched Drawing Routines
	// 16-bit Drawing Routines
	static int DrawPixel16 (int x, int y, int r, int g, int b);	// Draw pixel (16-bit)
	static int DrawImage16 (int x, int y, ImageX *img);			// Draw image (16-bit)
	static int DrawMasked16 (int x, int y, ImageX *img);		// Draw masked image (16-bit)
	static int DrawAlpha16 (int x, int y, ImageX *img);		// Draw alpha image (16-bit)
	static int DrawAdded16 (int x, int y, ImageX *img);		// Draw added image (16-bit)
	
	static int DrawImage24 (int x, int y, ImageX *img);	// Draw pixel (16-bit)
	static int DrawMasked24 (int x, int y, ImageX *img);			// Draw image (16-bit)
	static int DrawAlpha24 (int x, int y, ImageX *img);		// Draw masked image (16-bit)
	

	#ifdef GAMEX_MAIN
		WindowsDX GameX;
		int win_maskr,	win_maskg,	win_maskb;
		int win_shiftr, win_shiftg, win_shiftb;
		int win_maxr,	win_maxg,	win_maxb;
	#else
		extern WindowsDX GameX;
		extern int win_maskr,	win_maskg,	win_maskb;
		extern int win_shiftr,	win_shiftg, win_shiftb;
		extern int win_maxr,	win_maxg,	win_maxb;		
		extern void initialize (void);
		extern void main (void);
	#endif


#endif

